"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mode_1 = require("./mode");
const parsing_1 = require("./parsing");
const ENV_VAR_PREFIX = "USERSCRIPTER_";
exports.HOSTED_AT_EXAMPLE = "https://example.com/userscripts";
const ENVIRONMENT_VARIABLES = [
    {
        nameWithoutPrefix: "MODE",
        parser: parsing_1.enumParser(mode_1.isMode),
        overrides: "mode",
        mustBe: Object.values(mode_1.Mode),
    },
    {
        nameWithoutPrefix: "NIGHTLY",
        parser: parsing_1.booleanParser,
        overrides: "nightly",
        mustBe: ["true", "false"],
    },
    {
        nameWithoutPrefix: "HOSTED_AT",
        parser: parsing_1.urlParser,
        overrides: "hostedAt",
        mustBe: `a valid URL (e.g. "${exports.HOSTED_AT_EXAMPLE}")`,
    },
    {
        nameWithoutPrefix: "VERBOSE",
        parser: parsing_1.booleanParser,
        overrides: "verbose",
        mustBe: ["true", "false"],
    },
];
{
    const typecheckedEnvVars = ENVIRONMENT_VARIABLES;
    void typecheckedEnvVars;
}
function distFileName(id, type) {
    return [id, type, "js"].join(".");
}
exports.distFileName = distFileName;
function metadataUrl(hostedAt, id, type) {
    return hostedAt.replace(/\/?$/, "/") + distFileName(id, type);
}
exports.metadataUrl = metadataUrl;
function envVars(env) {
    return ENVIRONMENT_VARIABLES.map(e => {
        const name = ENV_VAR_PREFIX + e.nameWithoutPrefix;
        return [name, env[name]];
    });
}
exports.envVars = envVars;
function overrideBuildConfig(buildConfig, env) {
    return ENVIRONMENT_VARIABLES.reduce((acc, envVar) => {
        const envVarNameWithPrefix = ENV_VAR_PREFIX + envVar.nameWithoutPrefix;
        const parsed = fromEnv(envVar, env[envVarNameWithPrefix]);
        switch (parsed.kind) {
            case "undefined":
                return acc;
            case "valid":
                return {
                    ...acc,
                    buildConfig: {
                        ...acc.buildConfig,
                        [envVar.overrides]: parsed.value
                    },
                };
            case "invalid":
                return {
                    ...acc,
                    errors: acc.errors.concat({
                        fullName: envVarNameWithPrefix,
                        expected: envVar.mustBe,
                        found: parsed.input
                    }),
                };
        }
    }, { buildConfig, errors: [] });
}
exports.overrideBuildConfig = overrideBuildConfig;
function fromEnv(envVarSpec, v) {
    return (v === undefined
        ? { kind: "undefined" }
        : (envVarSpec.parser(v)));
}
