import { Mode, isMode } from "./mode";
import { booleanParser, enumParser, urlParser, } from "./parsing";
const ENV_VAR_PREFIX = "USERSCRIPTER_";
export const HOSTED_AT_EXAMPLE = "https://example.com/userscripts";
const ENVIRONMENT_VARIABLES = [
    {
        nameWithoutPrefix: "MODE",
        parser: enumParser(isMode),
        overrides: "mode",
        mustBe: Object.values(Mode),
    },
    {
        nameWithoutPrefix: "NIGHTLY",
        parser: booleanParser,
        overrides: "nightly",
        mustBe: ["true", "false"],
    },
    {
        nameWithoutPrefix: "HOSTED_AT",
        parser: urlParser,
        overrides: "hostedAt",
        mustBe: `a valid URL (e.g. "${HOSTED_AT_EXAMPLE}")`,
    },
    {
        nameWithoutPrefix: "VERBOSE",
        parser: booleanParser,
        overrides: "verbose",
        mustBe: ["true", "false"],
    },
];
{
    const typecheckedEnvVars = ENVIRONMENT_VARIABLES;
    void typecheckedEnvVars;
}
export function distFileName(id, type) {
    return [id, type, "js"].join(".");
}
export function metadataUrl(hostedAt, id, type) {
    return hostedAt.replace(/\/?$/, "/") + distFileName(id, type);
}
export function envVars(env) {
    return ENVIRONMENT_VARIABLES.map(e => {
        const name = ENV_VAR_PREFIX + e.nameWithoutPrefix;
        return [name, env[name]];
    });
}
export function overrideBuildConfig(buildConfig, env) {
    return ENVIRONMENT_VARIABLES.reduce((acc, envVar) => {
        const envVarNameWithPrefix = ENV_VAR_PREFIX + envVar.nameWithoutPrefix;
        const parsed = fromEnv(envVar, env[envVarNameWithPrefix]);
        switch (parsed.kind) {
            case "undefined":
                return acc;
            case "valid":
                return {
                    ...acc,
                    buildConfig: {
                        ...acc.buildConfig,
                        [envVar.overrides]: parsed.value
                    },
                };
            case "invalid":
                return {
                    ...acc,
                    errors: acc.errors.concat({
                        fullName: envVarNameWithPrefix,
                        expected: envVar.mustBe,
                        found: parsed.input
                    }),
                };
        }
    }, { buildConfig, errors: [] });
}
function fromEnv(envVarSpec, v) {
    return (v === undefined
        ? { kind: "undefined" }
        : (envVarSpec.parser(v)));
}
