"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_sass_1 = __importDefault(require("node-sass"));
const node_sass_utils_1 = __importDefault(require("node-sass-utils"));
const ts_type_guards_1 = require("ts-type-guards");
const SassUtils = node_sass_utils_1.default(node_sass_1.default);
function getGlobalFrom(objectToBeExposedToSass) {
    const sassVars = toSassDimension_recursively(objectToBeExposedToSass);
    return keyString => {
        if (keyString.getValue === undefined)
            fail("nothing");
        const wholeName = keyString.getValue();
        if (!ts_type_guards_1.isString(wholeName))
            return fail(wholeName);
        return SassUtils.castToSass(dig(sassVars, wholeName.split("."), wholeName));
    };
}
exports.getGlobalFrom = getGlobalFrom;
function toSassDimension(s) {
    const CSS_UNITS = ["rem", "em", "vh", "vw", "vmin", "vmax", "ex", "%", "px", "cm", "mm", "in", "pt", "pc", "ch"];
    const parts = s.match(/^([\.0-9]+)([a-zA-Z]+)$/);
    if (parts === null) {
        return s;
    }
    const number = parts[1];
    const unit = parts[2];
    if (CSS_UNITS.includes(unit)) {
        return new SassUtils.SassDimension(parseInt(number, 10), unit);
    }
    return s;
}
function toSassDimension_recursively(x) {
    if (ts_type_guards_1.isString(x)) {
        return toSassDimension(x);
    }
    else if (typeof x === "object") {
        const result = {};
        Object.keys(x).forEach(key => {
            result[key] = toSassDimension_recursively(x[key]);
        });
        return result;
    }
    else {
        return x;
    }
}
function fail(input) {
    throw new TypeError(`Expected a string, but saw: ${input}`);
}
function dig(obj, keys, wholeName) {
    if (keys.length === 0) {
        return obj;
    }
    else {
        const deeper = obj[keys[0]];
        if (deeper === undefined) {
            throw new Error(`Unknown global: '${wholeName}' (failed on '${keys[0]}')`);
        }
        return dig(deeper, keys.slice(1), wholeName);
    }
}
