"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const compose_1 = require("@typed/compose");
const Metadata = __importStar(require("userscript-metadata"));
const configuration_1 = require("./configuration");
const Msg = __importStar(require("./messages"));
const validation_1 = require("./validation");
class UserscripterWebpackPlugin {
    constructor(w) {
        this.w = w;
    }
    apply(compiler) {
        const w = this.w;
        compiler.hooks.afterCompile.tap(UserscripterWebpackPlugin.name, compilation => {
            const overridden = configuration_1.overrideBuildConfig(w.buildConfig, w.env);
            compilation.errors.push(...overridden.errors.map(compose_1.compose(Error, Msg.envVarError)));
            const configErrors = validation_1.buildConfigErrors(overridden.buildConfig);
            compilation.errors.push(...configErrors.map(compose_1.compose(Error, Msg.buildConfigError)));
            const metadataValidationResult = Metadata.validateWith(w.metadataSchema)(w.metadata(w.buildConfig));
            if (Metadata.isLeft(metadataValidationResult)) {
                compilation.errors.push(...metadataValidationResult.Left.map(compose_1.compose(Error, Msg.metadataError)));
            }
            else {
                compilation.warnings.push(...metadataValidationResult.Right.warnings.map(Msg.metadataWarning));
            }
        });
    }
}
exports.UserscripterWebpackPlugin = UserscripterWebpackPlugin;
