import { compose } from "@typed/compose";
import * as Metadata from "userscript-metadata";
import { overrideBuildConfig } from "./configuration";
import * as Msg from "./messages";
import { buildConfigErrors } from "./validation";
export class UserscripterWebpackPlugin {
    constructor(w) {
        this.w = w;
    }
    apply(compiler) {
        const w = this.w;
        compiler.hooks.afterCompile.tap(UserscripterWebpackPlugin.name, compilation => {
            const overridden = overrideBuildConfig(w.buildConfig, w.env);
            compilation.errors.push(...overridden.errors.map(compose(Error, Msg.envVarError)));
            const configErrors = buildConfigErrors(overridden.buildConfig);
            compilation.errors.push(...configErrors.map(compose(Error, Msg.buildConfigError)));
            const metadataValidationResult = Metadata.validateWith(w.metadataSchema)(w.metadata(w.buildConfig));
            if (Metadata.isLeft(metadataValidationResult)) {
                compilation.errors.push(...metadataValidationResult.Left.map(compose(Error, Msg.metadataError)));
            }
            else {
                compilation.warnings.push(...metadataValidationResult.Right.warnings.map(Msg.metadataWarning));
            }
        });
    }
}
